@@ -25,8 +25,6 @@
 namespace MiniZinc {
 
 ASTString op_to_builtin(EnvI& env, Expression* op_lhs, Expression* op_rhs, BinOpType bot) {
-  std::string builtin;
-
   if (Expression::type(op_rhs).st() == Type::ST_SET) {
     switch (bot) {
       case BOT_LE:
@@ -1114,6 +1112,10 @@ bool check_struct_table(EnvI& env, Type elem_t, Type arr_t) {
       // TODO: Can be filtered if float (because it must be par) or interned if int.
       return false;
     }
+    if (t.isOpt()) {
+      // TODO: Absent can be interned
+      return false;
+    }
   }
   return true;
 }
