@@ -65,9 +65,10 @@ OptimizeRegistry::ConstraintStatus o_linear(EnvI& env, Item* ii, Call* c, Expres
   }
   if (coeffs.size() == 1 && (ii->isa<ConstraintI>() || ii->cast<VarDeclI>()->e()->ti()->domain() ==
                                                            env.constants.literalTrue)) {
-    VarDecl* vd = x[0]()->cast<Id>()->decl();
+    VarDecl* vd = Expression::cast<Id>(x[0]())->decl();
     IntSetVal* domain =
         vd->ti()->domain() != nullptr ? eval_intset(env, vd->ti()->domain()) : nullptr;
+    assert(!domain->empty());
     if (c->id() == env.constants.ids.int_.lin_eq) {
       IntVal rd = eval_int(env, c->arg(2)) - d;
       if (rd % coeffs[0] == 0) {
@@ -78,7 +79,7 @@ OptimizeRegistry::ConstraintStatus o_linear(EnvI& env, Item* ii, Call* c, Expres
         std::vector<Expression*> args(2);
         args[0] = x[0]();
         args[1] = IntLit::a(nd);
-        Call* nc = new Call(Location(), env.constants.ids.int_.eq, args);
+        Call* nc = Call::a(Location(), env.constants.ids.int_.eq, args);
         nc->type(Type::varbool());
         rewrite = nc;
         return OptimizeRegistry::CS_REWRITE;
@@ -127,7 +128,7 @@ OptimizeRegistry::ConstraintStatus o_linear(EnvI& env, Item* ii, Call* c, Expres
         if (swapSign) {
           std::swap(args[0], args[1]);
         }
-        Call* nc = new Call(Location(), env.constants.ids.int_.le, args);
+        Call* nc = Call::a(Location(), env.constants.ids.int_.le, args);
         nc->type(Type::varbool());
         rewrite = nc;
         return OptimizeRegistry::CS_REWRITE;
@@ -139,7 +140,7 @@ OptimizeRegistry::ConstraintStatus o_linear(EnvI& env, Item* ii, Call* c, Expres
     std::vector<Expression*> args(2);
     args[0] = x[0]();
     args[1] = x[1]();
-    Call* nc = new Call(Location(), env.constants.ids.int_.eq, args);
+    Call* nc = Call::a(Location(), env.constants.ids.int_.eq, args);
     rewrite = nc;
     return OptimizeRegistry::CS_REWRITE;
   }
@@ -150,19 +151,19 @@ OptimizeRegistry::ConstraintStatus o_linear(EnvI& env, Item* ii, Call* c, Expres
       coeffs_e[i] = IntLit::a(coeffs[i]);
       x_e[i] = x[i]();
     }
-    auto* al_c_new = new ArrayLit(al_c->loc(), coeffs_e);
+    auto* al_c_new = new ArrayLit(Expression::loc(al_c), coeffs_e);
     al_c_new->type(Type::parint(1));
-    auto* al_x_new = new ArrayLit(al_x->loc(), x_e);
+    auto* al_x_new = new ArrayLit(Expression::loc(al_x), x_e);
     al_x_new->type(al_x->type());
 
     std::vector<Expression*> args(3);
     args[0] = al_c_new;
     args[1] = al_x_new;
     args[2] = IntLit::a(eval_int(env, c->arg(2)) - d);
-    Call* nc = new Call(Location(), c->id(), args);
+    Call* nc = Call::a(Location(), c->id(), args);
     nc->type(Type::varbool());
-    for (ExpressionSetIter it = c->ann().begin(); it != c->ann().end(); ++it) {
-      nc->addAnnotation(*it);
+    for (ExpressionSetIter it = Expression::ann(c).begin(); it != Expression::ann(c).end(); ++it) {
+      Expression::addAnnotation(nc, *it);
     }
 
     rewrite = nc;
@@ -201,19 +202,20 @@ OptimizeRegistry::ConstraintStatus o_lin_exp(EnvI& env, Item* i, Call* c, Expres
         coeffs_e[j] = IntLit::a(coeffs[j]);
         x_e[j] = x[j]();
       }
-      auto* al_c_new = new ArrayLit(al_c->loc(), coeffs_e);
+      auto* al_c_new = new ArrayLit(Expression::loc(al_c), coeffs_e);
       al_c_new->type(Type::parint(1));
-      auto* al_x_new = new ArrayLit(al_x->loc(), x_e);
+      auto* al_x_new = new ArrayLit(Expression::loc(al_x), x_e);
       al_x_new->type(al_x->type());
 
       std::vector<Expression*> args(3);
       args[0] = al_c_new;
       args[1] = al_x_new;
       args[2] = IntLit::a(d);
-      Call* nc = new Call(Location(), c->id(), args);
+      Call* nc = Call::a(Location(), c->id(), args);
       nc->type(c->type());
-      for (ExpressionSetIter it = c->ann().begin(); it != c->ann().end(); ++it) {
-        nc->addAnnotation(*it);
+      for (ExpressionSetIter it = Expression::ann(c).begin(); it != Expression::ann(c).end();
+           ++it) {
+        Expression::addAnnotation(nc, *it);
       }
       rewrite = nc;
       return OptimizeRegistry::CS_REWRITE;
@@ -223,7 +225,7 @@ OptimizeRegistry::ConstraintStatus o_lin_exp(EnvI& env, Item* i, Call* c, Expres
 }
 
 OptimizeRegistry::ConstraintStatus o_element(EnvI& env, Item* i, Call* c, Expression*& rewrite) {
-  if (c->arg(0)->isa<IntLit>()) {
+  if (Expression::isa<IntLit>(c->arg(0))) {
     IntVal idx = eval_int(env, c->arg(0));
     ArrayLit* al = eval_array_lit(env, c->arg(1));
     if (idx < 1 || idx > al->size()) {
@@ -233,7 +235,7 @@ OptimizeRegistry::ConstraintStatus o_element(EnvI& env, Item* i, Call* c, Expres
     std::vector<Expression*> args(2);
     args[0] = result;
     args[1] = c->arg(2);
-    Call* eq = new Call(Location(), env.constants.ids.int_.eq, args);
+    Call* eq = Call::a(Location(), env.constants.ids.int_.eq, args);
     rewrite = eq;
     return OptimizeRegistry::CS_REWRITE;
   }
@@ -245,7 +247,7 @@ OptimizeRegistry::ConstraintStatus o_clause(EnvI& env, Item* i, Call* c, Express
   std::vector<VarDecl*> neg;
   ArrayLit* al_pos = eval_array_lit(env, c->arg(0));
   for (unsigned int j = 0; j < al_pos->size(); j++) {
-    if (Id* ident = (*al_pos)[j]->dynamicCast<Id>()) {
+    if (Id* ident = Expression::dynamicCast<Id>((*al_pos)[j])) {
       if (ident->decl()->ti()->domain() == nullptr) {
         pos.push_back(ident->decl());
       }
@@ -253,7 +255,7 @@ OptimizeRegistry::ConstraintStatus o_clause(EnvI& env, Item* i, Call* c, Express
   }
   ArrayLit* al_neg = eval_array_lit(env, c->arg(1));
   for (unsigned int j = 0; j < al_neg->size(); j++) {
-    if (Id* ident = (*al_neg)[j]->dynamicCast<Id>()) {
+    if (Id* ident = Expression::dynamicCast<Id>((*al_neg)[j])) {
       if (ident->decl()->ti()->domain() == nullptr) {
         neg.push_back(ident->decl());
       }
@@ -286,20 +288,72 @@ OptimizeRegistry::ConstraintStatus o_clause(EnvI& env, Item* i, Call* c, Express
   return OptimizeRegistry::CS_OK;
 }
 
+OptimizeRegistry::ConstraintStatus o_forall(EnvI& env, Item* i, Call* c, Expression*& rewrite) {
+  ArrayLit* al = eval_array_lit(env, c->arg(0));
+  bool subsumed = true;
+  for (unsigned int j = 0; j < al->size(); j++) {
+    if (Expression::type((*al)[j]).isPar()) {
+      if (!eval_bool(env, (*al)[j])) {
+        return OptimizeRegistry::CS_FAILED;
+      }
+    } else if (Id* ident = Expression::dynamicCast<Id>((*al)[j])) {
+      if (Expression* dom = ident->decl()->ti()->domain()) {
+        if (dom == env.constants.literalFalse) {
+          return OptimizeRegistry::CS_FAILED;
+        }
+      } else {
+        subsumed = false;
+      }
+    } else {
+      subsumed = false;
+    }
+  }
+  if (subsumed) {
+    return OptimizeRegistry::CS_ENTAILED;
+  }
+  return OptimizeRegistry::CS_OK;
+}
+
+OptimizeRegistry::ConstraintStatus o_exists(EnvI& env, Item* i, Call* c, Expression*& rewrite) {
+  ArrayLit* al = eval_array_lit(env, c->arg(0));
+  bool failed = true;
+  for (unsigned int j = 0; j < al->size(); j++) {
+    if (Expression::type((*al)[j]).isPar()) {
+      if (eval_bool(env, (*al)[j])) {
+        return OptimizeRegistry::CS_ENTAILED;
+      }
+    } else if (Id* ident = Expression::dynamicCast<Id>((*al)[j])) {
+      if (Expression* dom = ident->decl()->ti()->domain()) {
+        if (dom == env.constants.literalTrue) {
+          return OptimizeRegistry::CS_ENTAILED;
+        }
+      } else {
+        failed = false;
+      }
+    } else {
+      failed = false;
+    }
+  }
+  if (failed) {
+    return OptimizeRegistry::CS_FAILED;
+  }
+  return OptimizeRegistry::CS_OK;
+}
+
 OptimizeRegistry::ConstraintStatus o_not(EnvI& env, Item* i, Call* c, Expression*& rewrite) {
   if (c->argCount() == 2) {
     Expression* e0 = c->arg(0);
     Expression* e1 = c->arg(1);
-    if (e0->type().isPar() && e1->type().isPar()) {
+    if (Expression::type(e0).isPar() && Expression::type(e1).isPar()) {
       return eval_bool(env, e0) == eval_bool(env, e1) ? OptimizeRegistry::CS_FAILED
                                                       : OptimizeRegistry::CS_ENTAILED;
     }
-    if (e1->type().isPar()) {
+    if (Expression::type(e1).isPar()) {
       std::swap(e0, e1);
     }
-    if (e0->type().isPar()) {
-      Call* eq = new Call(Location(), env.constants.ids.bool_.eq,
-                          {e1, env.constants.boollit(!eval_bool(env, e0))});
+    if (Expression::type(e0).isPar()) {
+      Call* eq = Call::a(Location(), env.constants.ids.bool_.eq,
+                         {e1, env.constants.boollit(!eval_bool(env, e0))});
       rewrite = eq;
       return OptimizeRegistry::CS_REWRITE;
     }
@@ -308,9 +362,10 @@ OptimizeRegistry::ConstraintStatus o_not(EnvI& env, Item* i, Call* c, Expression
 }
 
 OptimizeRegistry::ConstraintStatus o_div(EnvI& env, Item* i, Call* c, Expression*& rewrite) {
-  if (c->arg(1)->type().isPar()) {
+  if (Expression::type(c->arg(1)).isPar()) {
     IntVal c1v = eval_int(env, c->arg(1));
-    if (c->arg(0)->type().isPar() && c->argCount() == 3 && c->arg(2)->type().isPar()) {
+    if (Expression::type(c->arg(0)).isPar() && c->argCount() == 3 &&
+        Expression::type(c->arg(2)).isPar()) {
       IntVal c0v = eval_int(env, c->arg(0));
       IntVal c2v = eval_int(env, c->arg(2));
       return (c0v / c1v == c2v) ? OptimizeRegistry::CS_ENTAILED : OptimizeRegistry::CS_FAILED;
@@ -323,18 +378,18 @@ OptimizeRegistry::ConstraintStatus o_times(EnvI& env, Item* i, Call* c, Expressi
   Expression* result = nullptr;
   Expression* arg0 = c->arg(0);
   Expression* arg1 = c->arg(1);
-  if (arg0->type().isPar() && arg1->type().isPar()) {
+  if (Expression::type(arg0).isPar() && Expression::type(arg1).isPar()) {
     IntVal c0v = eval_int(env, arg0);
     IntVal c1v = eval_int(env, arg1);
     result = IntLit::a(c0v * c1v);
-  } else if (arg0->type().isPar()) {
+  } else if (Expression::type(arg0).isPar()) {
     IntVal c0v = eval_int(env, arg0);
     if (c0v == 0) {
       result = IntLit::a(0);
     } else if (c0v == 1) {
       result = arg1;
     }
-  } else if (arg1->type().isPar()) {
+  } else if (Expression::type(arg1).isPar()) {
     IntVal c1v = eval_int(env, arg1);
     if (c1v == 0) {
       result = IntLit::a(0);
@@ -351,20 +406,20 @@ OptimizeRegistry::ConstraintStatus o_times(EnvI& env, Item* i, Call* c, Expressi
       return OptimizeRegistry::CS_REWRITE;
     }  // this is the relational version of times
     assert(c->argCount() == 3);
-    rewrite = new Call(Location().introduce(), env.constants.ids.int_.eq, {c->arg(2), result});
+    rewrite = Call::a(Location().introduce(), env.constants.ids.int_.eq, {c->arg(2), result});
     return OptimizeRegistry::CS_REWRITE;
   }
   return OptimizeRegistry::CS_OK;
 }
 
 OptimizeRegistry::ConstraintStatus o_set_in(EnvI& env, Item* i, Call* c, Expression*& rewrite) {
-  if (c->arg(1)->type().isPar()) {
-    if (c->arg(0)->type().isPar()) {
+  if (Expression::type(c->arg(1)).isPar()) {
+    if (Expression::type(c->arg(0)).isPar()) {
       IntSetVal* isv = eval_intset(env, c->arg(1));
       return isv->contains(eval_int(env, c->arg(0))) ? OptimizeRegistry::CS_ENTAILED
                                                      : OptimizeRegistry::CS_FAILED;
     }
-    if (Id* ident = c->arg(0)->dynamicCast<Id>()) {
+    if (Id* ident = Expression::dynamicCast<Id>(c->arg(0))) {
       VarDecl* vd = ident->decl();
       IntSetVal* isv = eval_intset(env, c->arg(1));
       if (vd->ti()->domain() != nullptr) {
@@ -387,7 +442,7 @@ OptimizeRegistry::ConstraintStatus o_set_in(EnvI& env, Item* i, Call* c, Express
         std::vector<Expression*> args(2);
         args[0] = vd->id();
         args[1] = IntLit::a(isv->min());
-        Call* eq = new Call(Location(), env.constants.ids.int_.eq, args);
+        Call* eq = Call::a(Location(), env.constants.ids.int_.eq, args);
         rewrite = eq;
         return OptimizeRegistry::CS_REWRITE;
       }
@@ -399,15 +454,15 @@ OptimizeRegistry::ConstraintStatus o_set_in(EnvI& env, Item* i, Call* c, Express
 OptimizeRegistry::ConstraintStatus o_int_ne(EnvI& env, Item* i, Call* c, Expression*& rewrite) {
   Expression* e0 = c->arg(0);
   Expression* e1 = c->arg(1);
-  if (e0->type().isPar() && e1->type().isPar()) {
+  if (Expression::type(e0).isPar() && Expression::type(e1).isPar()) {
     return eval_int(env, e0) != eval_int(env, e1) ? OptimizeRegistry::CS_ENTAILED
                                                   : OptimizeRegistry::CS_FAILED;
   }
-  if (e1->isa<Id>()) {
+  if (Expression::isa<Id>(e1)) {
     std::swap(e0, e1);
   }
-  if (Id* ident = e0->dynamicCast<Id>()) {
-    if (e1->type().isPar()) {
+  if (Id* ident = Expression::dynamicCast<Id>(e0)) {
+    if (Expression::type(e1).isPar()) {
       if (ident->decl()->ti()->domain() != nullptr) {
         IntVal e1v = eval_int(env, e1);
         IntSetVal* isv = eval_intset(env, ident->decl()->ti()->domain());
@@ -427,17 +482,17 @@ OptimizeRegistry::ConstraintStatus o_int_ne(EnvI& env, Item* i, Call* c, Express
 OptimizeRegistry::ConstraintStatus o_int_le(EnvI& env, Item* i, Call* c, Expression*& rewrite) {
   Expression* e0 = c->arg(0);
   Expression* e1 = c->arg(1);
-  if (e0->type().isPar() && e1->type().isPar()) {
+  if (Expression::type(e0).isPar() && Expression::type(e1).isPar()) {
     return eval_int(env, e0) <= eval_int(env, e1) ? OptimizeRegistry::CS_ENTAILED
                                                   : OptimizeRegistry::CS_FAILED;
   }
   bool swapped = false;
-  if (e1->isa<Id>()) {
+  if (Expression::isa<Id>(e1)) {
     std::swap(e0, e1);
     swapped = true;
   }
-  if (Id* ident = e0->dynamicCast<Id>()) {
-    if (e1->type().isPar()) {
+  if (Id* ident = Expression::dynamicCast<Id>(e0)) {
+    if (Expression::type(e1).isPar()) {
       if (ident->decl()->ti()->domain() != nullptr) {
         IntVal e1v = eval_int(env, e1);
         IntSetVal* isv = eval_intset(env, ident->decl()->ti()->domain());
@@ -487,6 +542,8 @@ public:
     OptimizeRegistry::registry().reg(id_var_element, o_element);
     OptimizeRegistry::registry().reg(Constants::constants().ids.clause, o_clause);
     OptimizeRegistry::registry().reg(Constants::constants().ids.bool_.clause, o_clause);
+    OptimizeRegistry::registry().reg(Constants::constants().ids.forall, o_forall);
+    OptimizeRegistry::registry().reg(Constants::constants().ids.exists, o_exists);
     OptimizeRegistry::registry().reg(Constants::constants().ids.bool_.not_, o_not);
     OptimizeRegistry::registry().reg(Constants::constants().ids.set_.in, o_set_in);
     OptimizeRegistry::registry().reg(Constants::constants().ids.int_.ne, o_int_ne);
