@@ -985,7 +985,6 @@ void MznSolver::flatten(const std::string& modelString, const std::string& model
     }
     return time_left.count();
   });
-  auto timer_future = timer.get_future();
   std::thread thr(std::move(timer));
 #endif
 
@@ -1132,7 +1131,7 @@ SolverInstance::Status MznSolver::run(const std::vector<std::string>& args0,
   if (SolverInstance::UNKNOWN == getFltStatus()) {
     if (!ifMzn2Fzn()) {  // only then
       // Special handling of basic stdFlags
-      auto* solve_item = _flt.getEnv()->model()->solveItem();
+      auto* solve_item = _flt.getEnv()->flat()->solveItem();
       bool is_sat_problem =
           solve_item != nullptr ? solve_item->st() == SolveI::SolveType::ST_SAT : true;
       if (is_sat_problem && _flagAllSatisfaction) {
@@ -1160,6 +1159,7 @@ SolverInstance::Status MznSolver::run(const std::vector<std::string>& args0,
     return SolverInstance::NONE;
   }
   if (!ifMzn2Fzn()) {
+    addSolverInterface();
     s2out.evalStatus(getFltStatus());
   }
   return getFltStatus();
