@@ -82,7 +82,10 @@ void dll_close(void* dll) {
 #endif
 
 const vector<string>& cplex_dlls() {
-  static const vector<string> sCPLEXDLLs = {"cplex12100", "cplex1290", "cplex1280", "cplex1270"};
+  static const vector<string> sCPLEXDLLs = {
+      "cplex2412", "cplex2411",  "cplex2410", "cplex2312", "cplex2311", "cplex2310",
+      "cplex2222", "cplex2221",  "cplex2220", "cplex2212", "cplex2211", "cplex2210",
+      "cplex2010", "cplex12100", "cplex1290", "cplex1280", "cplex1270"};
   return sCPLEXDLLs;
 }
 
@@ -276,7 +279,7 @@ vector<string> MIPCplexWrapper::getRequiredFlags(FactoryOptions& f) {
     if (env != nullptr) {
       return {};
     }
-  } catch (MiniZinc::Error&) {
+  } catch (MiniZinc::Error&) { /* NOLINT(bugprone-empty-catch) */
   }
   return {"--cplex-dll"};
 }
@@ -571,7 +574,8 @@ bool MIPCplexWrapper::Options::processOption(int& i, std::vector<std::string>& a
   std::string buffer;
   if (cop.get("-i")) {
     flagIntermediate = true;
-  } else if (string(argv[i]) == "-f") {  // NOLINT: Allow repeated empty if
+  } else if (string(argv[i]) == "-f" ||
+             string(argv[i]) == "--free-search") {  // NOLINT: Allow repeated empty if
     //     std::cerr << "  Flag -f: ignoring fixed strategy anyway." << std::endl;
   } else if (cop.get("--mipfocus --mipFocus --MIPFocus --MIPfocus",
                      &nMIPFocus)) {  // NOLINT: Allow repeated empty if
