@@ -243,6 +243,7 @@ void GeasSolverInstance::processFlatZinc() {
           Expression* domain = vd->ti()->domain();
           if (domain != nullptr) {
             IntSetVal* isv = eval_intset(env().envi(), domain);
+            assert(!isv->empty());
             auto var = _solver.new_intvar(static_cast<geas::intvar::val_t>(isv->min().toInt()),
                                           static_cast<geas::intvar::val_t>(isv->max().toInt()));
             if (isv->size() > 1) {
@@ -729,7 +730,7 @@ bool GeasSolverFactory::processOption(SolverInstanceBase::Options* opt, int& i,
     if (nodes >= 0) {
       _opt->conflicts = nodes;
     }
-  } else if (argv[i] == "-f") {
+  } else if (argv[i] == "-f" || argv[i] == "--free-search") {
     _opt->freeSearch = true;
   } else if (argv[i] == "-n") {
     if (++i == argv.size()) {
