@@ -606,7 +606,7 @@ void p_array_bool_and_imp(SolverInstanceBase& s, const Call* call) {
   auto& gi = static_cast<GecodeSolverInstance&>(s);
   BoolVarArgs bv = gi.arg2boolvarargs(call->arg(0));
   BoolVar b1 = gi.arg2boolvar(call->arg(1));
-  for (int i = static_cast<int>(bv.size()); (i--) != 0;) {
+  for (int i = bv.size(); (i--) != 0;) {
     rel(*gi.currentSpace, b1, Gecode::BoolOpType::BOT_IMP, bv[i], 1,
         GecodeSolverInstance::ann2icl(ann));
   }
